"""custom enums to respect the xml schema."""

types = {
    "byte": "Byte",
    "uint16": "UInt16",
    "int16": "Int16",
    "uint32": "UInt32",
    "int32": "Int32",
    "uint64": "UInt64",
    "int64": "Int64",
    "float32": "Float32",
    "float63": "Float64",
    "cint16": "CInt16",
    "cint32": "CInt32",
    "cfloat32": "CFloat32",
    "cfloat64": "CFloat64",
}
"cooresponding XML types to python one"
